@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # Change the list according to your local conda/virtualenv env.
-CONDA_ENVS="py26-np16 py27-np17 py33-np18 py34-np19 py35-np19"
+CONDA_ENVS="py27-np16 py27-np17 py33-np18 py34-np19 py35-np19 py36-np111"
 COMPRESS_METHODS="zlib gzip bz2 xz lzma"
 
 for i in $CONDA_ENVS
