@@ -8,23 +8,26 @@ Test the parallel module.
 
 import time
 import sys
-import io
 import os
 from math import sqrt
+import threading
+from multiprocessing import TimeoutError
+from time import sleep
+import mmap
 
+from joblib import dump, load
 from joblib import parallel
 
 from joblib.test.common import np, with_numpy
 from joblib.test.common import with_multiprocessing
-from joblib.testing import check_subprocess_call
+from joblib.testing import (parametrize, raises, check_subprocess_call,
+                            SkipTest, warns)
 from joblib._compat import PY3_OR_LATER
-from multiprocessing import TimeoutError
-from time import sleep
 
 try:
     import cPickle as pickle
     PickleError = TypeError
-except:
+except ImportError:
     import pickle
     PickleError = pickle.PicklingError
 
@@ -44,6 +47,10 @@ except ImportError:
     # Backward compat
     from Queue import Queue
 
+try:
+    import posix
+except ImportError:
+    posix = None
 
 from joblib._parallel_backends import SequentialBackend
 from joblib._parallel_backends import ThreadingBackend
@@ -57,11 +64,10 @@ from joblib.parallel import register_parallel_backend, parallel_backend
 from joblib.parallel import mp, cpu_count, BACKENDS, effective_n_jobs
 from joblib.my_exceptions import JoblibException
 
-import nose
-from nose.tools import assert_equal, assert_true, assert_false, assert_raises
-
 
 ALL_VALID_BACKENDS = [None] + sorted(BACKENDS.keys())
+# Add instances of backend classes deriving from ParallelBackendBase
+ALL_VALID_BACKENDS += [BACKENDS[backend_str]() for backend_str in BACKENDS]
 
 if hasattr(mp, 'get_context'):
     # Custom multiprocessing context in Python 3.4+
@@ -106,6 +112,10 @@ def _active_backend_type():
     return type(parallel.get_active_backend()[0])
 
 
+def parallel_func(inner_n_jobs):
+    return Parallel(n_jobs=inner_n_jobs)(delayed(square)(i) for i in range(3))
+
+
 ###############################################################################
 def test_cpu_count():
     assert cpu_count() > 0
@@ -117,49 +127,34 @@ def test_effective_n_jobs():
 
 ###############################################################################
 # Test parallel
-def check_simple_parallel(backend):
-    X = range(5)
-    for n_jobs in (1, 2, -1, -2):
-        assert_equal(
-            [square(x) for x in X],
-            Parallel(n_jobs=n_jobs, backend=backend)(
-                delayed(square)(x) for x in X))
-    try:
-        # To smoke-test verbosity, we capture stdout
-        orig_stdout = sys.stdout
-        orig_stderr = sys.stdout
-        if PY3_OR_LATER:
-            sys.stderr = io.StringIO()
-            sys.stderr = io.StringIO()
-        else:
-            sys.stdout = io.BytesIO()
-            sys.stderr = io.BytesIO()
-        for verbose in (2, 11, 100):
-            Parallel(n_jobs=-1, verbose=verbose, backend=backend)(
-                delayed(square)(x) for x in X)
-            Parallel(n_jobs=1, verbose=verbose, backend=backend)(
-                delayed(square)(x) for x in X)
-            Parallel(n_jobs=2, verbose=verbose, pre_dispatch=2,
-                     backend=backend)(
-                delayed(square)(x) for x in X)
-            Parallel(n_jobs=2, verbose=verbose, backend=backend)(
-                delayed(square)(x) for x in X)
-    except Exception as e:
-        my_stdout = sys.stdout
-        my_stderr = sys.stderr
-        sys.stdout = orig_stdout
-        sys.stderr = orig_stderr
-        print(unicode(my_stdout.getvalue()))
-        print(unicode(my_stderr.getvalue()))
-        raise e
-    finally:
-        sys.stdout = orig_stdout
-        sys.stderr = orig_stderr
+
+@parametrize('backend', ALL_VALID_BACKENDS)
+@parametrize('n_jobs', [1, 2, -1, -2])
+@parametrize('verbose', [2, 11, 100])
+def test_simple_parallel(backend, n_jobs, verbose):
+    assert ([square(x) for x in range(5)] ==
+            Parallel(n_jobs=n_jobs, backend=backend,
+                     verbose=verbose)(
+                delayed(square)(x) for x in range(5)))
 
 
-def test_simple_parallel():
-    for backend in ALL_VALID_BACKENDS:
-        yield check_simple_parallel, backend
+@parametrize('backend', ALL_VALID_BACKENDS)
+def test_main_thread_renamed_no_warning(backend, monkeypatch):
+    # Check that no default backend relies on the name of the main thread:
+    # https://github.com/joblib/joblib/issues/180#issuecomment-253266247
+    # Some programs use a different name for the main thread. This is the case
+    # for uWSGI apps for instance.
+    monkeypatch.setattr(target=threading.current_thread(), name='name',
+                        value='some_new_name_for_the_main_thread')
+
+    with warns(None) as warninfo:
+        results = Parallel(n_jobs=2, backend=backend)(
+            delayed(square)(x) for x in range(3))
+        assert results == [0, 1, 4]
+    # The multiprocessing backend will raise a warning when detecting that is
+    # started from the non-main thread. Let's check that there is no false
+    # positive because of the name change.
+    assert len(warninfo) == 0
 
 
 def nested_loop(backend):
@@ -167,35 +162,31 @@ def nested_loop(backend):
         delayed(square)(.01) for _ in range(2))
 
 
-def check_nested_loop(parent_backend, child_backend):
+@parametrize('parent_backend', BACKENDS)
+@parametrize('child_backend', BACKENDS)
+def test_nested_loop(parent_backend, child_backend):
     Parallel(n_jobs=2, backend=parent_backend)(
         delayed(nested_loop)(child_backend) for _ in range(2))
 
 
-def test_nested_loop():
-    for parent_backend in BACKENDS:
-        for child_backend in BACKENDS:
-            yield check_nested_loop, parent_backend, child_backend
-
-
 def test_mutate_input_with_threads():
     """Input is mutable when using the threading backend"""
     q = Queue(maxsize=5)
     Parallel(n_jobs=2, backend="threading")(
         delayed(q.put, check_pickle=False)(1) for _ in range(5))
-    assert_true(q.full())
+    assert q.full()
 
 
-def test_parallel_kwargs():
+@parametrize('n_jobs', [1, 2, 3])
+def test_parallel_kwargs(n_jobs):
     """Check the keyword argument processing of pmap."""
     lst = range(10)
-    for n_jobs in (1, 4):
-        yield (assert_equal,
-               [f(x, y=1) for x in lst],
-               Parallel(n_jobs=n_jobs)(delayed(f)(x, y=1) for x in lst))
+    assert ([f(x, y=1) for x in lst] ==
+            Parallel(n_jobs=n_jobs)(delayed(f)(x, y=1) for x in lst))
 
 
-def check_parallel_as_context_manager(backend):
+@parametrize('backend', ['multiprocessing', 'threading'])
+def test_parallel_as_context_manager(backend):
     lst = range(10)
     expected = [f(x, y=1) for x in lst]
     with Parallel(n_jobs=4, backend=backend) as p:
@@ -203,32 +194,27 @@ def check_parallel_as_context_manager(backend):
         # via the context manager protocol
         managed_backend = p._backend
         if mp is not None:
-            assert_true(managed_backend is not None)
-            assert_true(managed_backend._pool is not None)
+            assert managed_backend is not None
+            assert managed_backend._pool is not None
 
         # We make call with the managed parallel object several times inside
         # the managed block:
-        assert_equal(expected, p(delayed(f)(x, y=1) for x in lst))
-        assert_equal(expected, p(delayed(f)(x, y=1) for x in lst))
+        assert expected == p(delayed(f)(x, y=1) for x in lst)
+        assert expected == p(delayed(f)(x, y=1) for x in lst)
 
         # Those calls have all used the same pool instance:
         if mp is not None:
-            assert_true(managed_backend._pool is p._backend._pool)
+            assert managed_backend._pool is p._backend._pool
 
     # As soon as we exit the context manager block, the pool is terminated and
     # no longer referenced from the parallel object:
     if mp is not None:
-        assert_true(p._backend._pool is None)
+        assert p._backend._pool is None
 
     # It's still possible to use the parallel instance in non-managed mode:
-    assert_equal(expected, p(delayed(f)(x, y=1) for x in lst))
+    assert expected == p(delayed(f)(x, y=1) for x in lst)
     if mp is not None:
-        assert_true(p._backend._pool is None)
-
-
-def test_parallel_context_manager():
-    for backend in ['multiprocessing', 'threading']:
-        yield check_parallel_as_context_manager, backend
+        assert p._backend._pool is None
 
 
 def test_parallel_pickling():
@@ -246,27 +232,24 @@ def test_parallel_pickling():
     except Exception as exc:
         exception_class = exc.__class__
 
-    assert_raises(exception_class, Parallel(),
-                  (delayed(g)(x) for x in range(10)))
+    with raises(exception_class):
+        Parallel()(delayed(g)(x) for x in range(10))
 
 
-def test_parallel_timeout_success():
+@parametrize('backend', ['multiprocessing', 'threading'])
+def test_parallel_timeout_success(backend):
     # Check that timeout isn't thrown when function is fast enough
-    for backend in ['multiprocessing', 'threading']:
-        nose.tools.assert_equal(
-            10,
-            len(Parallel(n_jobs=2, backend=backend, timeout=10)
-                (delayed(sleep)(0.001) for x in range(10))))
+    assert len(Parallel(n_jobs=2, backend=backend, timeout=10)(
+        delayed(sleep)(0.001) for x in range(10))) == 10
 
 
-def test_parallel_timeout_fail():
+@with_multiprocessing
+@parametrize('backend', ['multiprocessing', 'threading'])
+def test_parallel_timeout_fail(backend):
     # Check that timeout properly fails when function is too slow
-    for backend in ['multiprocessing', 'threading']:
-        nose.tools.assert_raises(
-            TimeoutError,
-            Parallel(n_jobs=2, backend=backend, timeout=0.01),
-            (delayed(sleep)(10) for x in range(10))
-        )
+    with raises(TimeoutError):
+        Parallel(n_jobs=2, backend=backend, timeout=0.01)(
+            delayed(sleep)(10) for x in range(10))
 
 
 def test_error_capture():
@@ -275,61 +258,64 @@ def test_error_capture():
     if mp is not None:
         # A JoblibException will be raised only if there is indeed
         # multiprocessing
-        assert_raises(JoblibException, Parallel(n_jobs=2),
-                      [delayed(division)(x, y)
-                       for x, y in zip((0, 1), (1, 0))])
-        assert_raises(WorkerInterrupt, Parallel(n_jobs=2),
-                      [delayed(interrupt_raiser)(x) for x in (1, 0)])
+        with raises(JoblibException):
+            Parallel(n_jobs=2)(
+                [delayed(division)(x, y)
+                    for x, y in zip((0, 1), (1, 0))])
+        with raises(WorkerInterrupt):
+            Parallel(n_jobs=2)(
+                [delayed(interrupt_raiser)(x) for x in (1, 0)])
 
         # Try again with the context manager API
         with Parallel(n_jobs=2) as parallel:
-            assert_true(parallel._backend._pool is not None)
+            assert parallel._backend._pool is not None
             original_pool = parallel._backend._pool
 
-            assert_raises(JoblibException, parallel,
-                          [delayed(division)(x, y)
-                           for x, y in zip((0, 1), (1, 0))])
+            with raises(JoblibException):
+                parallel([delayed(division)(x, y)
+                          for x, y in zip((0, 1), (1, 0))])
 
             # The managed pool should still be available and be in a working
             # state despite the previously raised (and caught) exception
-            assert_true(parallel._backend._pool is not None)
+            assert parallel._backend._pool is not None
 
             # The pool should have been interrupted and restarted:
-            assert_true(parallel._backend._pool is not original_pool)
+            assert parallel._backend._pool is not original_pool
 
-            assert_equal([f(x, y=1) for x in range(10)],
-                         parallel(delayed(f)(x, y=1) for x in range(10)))
+            assert ([f(x, y=1) for x in range(10)] ==
+                    parallel(delayed(f)(x, y=1) for x in range(10)))
 
             original_pool = parallel._backend._pool
-            assert_raises(WorkerInterrupt, parallel,
-                          [delayed(interrupt_raiser)(x) for x in (1, 0)])
+            with raises(WorkerInterrupt):
+                parallel([delayed(interrupt_raiser)(x) for x in (1, 0)])
 
             # The pool should still be available despite the exception
-            assert_true(parallel._backend._pool is not None)
+            assert parallel._backend._pool is not None
 
             # The pool should have been interrupted and restarted:
-            assert_true(parallel._backend._pool is not original_pool)
+            assert parallel._backend._pool is not original_pool
 
-            assert_equal([f(x, y=1) for x in range(10)],
-                         parallel(delayed(f)(x, y=1) for x in range(10)))
+            assert ([f(x, y=1) for x in range(10)] ==
+                    parallel(delayed(f)(x, y=1) for x in range(10)))
 
         # Check that the inner pool has been terminated when exiting the
         # context manager
-        assert_true(parallel._backend._pool is None)
+        assert parallel._backend._pool is None
     else:
-        assert_raises(KeyboardInterrupt, Parallel(n_jobs=2),
-                      [delayed(interrupt_raiser)(x) for x in (1, 0)])
+        with raises(KeyboardInterrupt):
+            Parallel(n_jobs=2)(
+                [delayed(interrupt_raiser)(x) for x in (1, 0)])
 
     # wrapped exceptions should inherit from the class of the original
     # exception to make it easy to catch them
-    assert_raises(ZeroDivisionError, Parallel(n_jobs=2),
-                  [delayed(division)(x, y) for x, y in zip((0, 1), (1, 0))])
+    with raises(ZeroDivisionError):
+        Parallel(n_jobs=2)(
+            [delayed(division)(x, y) for x, y in zip((0, 1), (1, 0))])
 
-    assert_raises(
-        MyExceptionWithFinickyInit,
-        Parallel(n_jobs=2, verbose=0),
-        (delayed(exception_raiser)(i, custom_exception=True)
-         for i in range(30)))
+    with raises(MyExceptionWithFinickyInit):
+        Parallel(n_jobs=2, verbose=0)(
+            (delayed(exception_raiser)(i, custom_exception=True)
+             for i in range(30)))
 
     try:
         # JoblibException wrapping is disabled in sequential mode:
@@ -337,24 +323,27 @@ def test_error_capture():
         Parallel(n_jobs=1)(
             delayed(division)(x, y) for x, y in zip((0, 1), (1, 0)))
     except Exception as ex:
-        assert_false(isinstance(ex, JoblibException))
-
-
-class Counter(object):
-    def __init__(self, list1, list2):
-        self.list1 = list1
-        self.list2 = list2
-
-    def __call__(self, i):
-        self.list1.append(i)
-        assert_equal(len(self.list1), len(self.list2))
+        assert not isinstance(ex, JoblibException)
 
 
 def consumer(queue, item):
     queue.append('Consumed %s' % item)
 
 
-def check_dispatch_one_job(backend):
+@parametrize('backend', BACKENDS)
+@parametrize('batch_size, expected_queue',
+             [(1, ['Produced 0', 'Consumed 0',
+                   'Produced 1', 'Consumed 1',
+                   'Produced 2', 'Consumed 2',
+                   'Produced 3', 'Consumed 3',
+                   'Produced 4', 'Consumed 4',
+                   'Produced 5', 'Consumed 5']),
+              (4, [  # First Batch
+                  'Produced 0', 'Produced 1', 'Produced 2', 'Produced 3',
+                  'Consumed 0', 'Consumed 1', 'Consumed 2', 'Consumed 3',
+                     # Second batch
+                  'Produced 4', 'Produced 5', 'Consumed 4', 'Consumed 5'])])
+def test_dispatch_one_job(backend, batch_size, expected_queue):
     """ Test that with only one job, Parallel does act as a iterator.
     """
     queue = list()
@@ -364,47 +353,18 @@ def check_dispatch_one_job(backend):
             queue.append('Produced %i' % i)
             yield i
 
-    # disable batching
-    Parallel(n_jobs=1, batch_size=1, backend=backend)(
+    Parallel(n_jobs=1, batch_size=batch_size, backend=backend)(
         delayed(consumer)(queue, x) for x in producer())
-    assert_equal(queue, [
-        'Produced 0', 'Consumed 0',
-        'Produced 1', 'Consumed 1',
-        'Produced 2', 'Consumed 2',
-        'Produced 3', 'Consumed 3',
-        'Produced 4', 'Consumed 4',
-        'Produced 5', 'Consumed 5',
-    ])
-    assert_equal(len(queue), 12)
-
-    # empty the queue for the next check
-    queue[:] = []
-
-    # enable batching
-    Parallel(n_jobs=1, batch_size=4, backend=backend)(
-        delayed(consumer)(queue, x) for x in producer())
-    assert_equal(queue, [
-        # First batch
-        'Produced 0', 'Produced 1', 'Produced 2', 'Produced 3',
-        'Consumed 0', 'Consumed 1', 'Consumed 2', 'Consumed 3',
-
-        # Second batch
-        'Produced 4', 'Produced 5', 'Consumed 4', 'Consumed 5',
-    ])
-    assert_equal(len(queue), 12)
+    assert queue == expected_queue
+    assert len(queue) == 12
 
 
-def test_dispatch_one_job():
-    for backend in BACKENDS:
-        yield check_dispatch_one_job, backend
-
-
-def check_dispatch_multiprocessing(backend):
+@with_multiprocessing
+@parametrize('backend', ['multiprocessing', 'threading'])
+def test_dispatch_multiprocessing(backend):
     """ Check that using pre_dispatch Parallel does indeed dispatch items
         lazily.
     """
-    if mp is None:
-        raise nose.SkipTest()
     manager = mp.Manager()
     queue = manager.list()
 
@@ -422,15 +382,8 @@ def check_dispatch_multiprocessing(backend):
     # The the first consumption event can sometimes happen before the end of
     # the dispatching, hence, pop it before introspecting the "Produced" events
     first_four.remove('Consumed any')
-    assert_equal(first_four,
-                 ['Produced 0', 'Produced 1', 'Produced 2'])
-    assert_equal(len(queue), 12)
-
-
-def test_dispatch_multiprocessing():
-    for backend in BACKENDS:
-        if backend != "sequential":
-            yield check_dispatch_multiprocessing, backend
+    assert first_four == ['Produced 0', 'Produced 1', 'Produced 2']
+    assert len(queue) == 12
 
 
 def test_batching_auto_threading():
@@ -440,7 +393,7 @@ def test_batching_auto_threading():
 
     with Parallel(n_jobs=2, batch_size='auto', backend='threading') as p:
         p(delayed(id)(i) for i in range(5000))  # many very fast tasks
-        assert_equal(p._backend.compute_batch_size(), 1)
+        assert p._backend.compute_batch_size() == 1
 
 
 def test_batching_auto_multiprocessing():
@@ -450,24 +403,22 @@ def test_batching_auto_multiprocessing():
         # It should be strictly larger than 1 but as we don't want heisen
         # failures on clogged CI worker environment be safe and only check that
         # it's a strictly positive number.
-        assert_true(p._backend.compute_batch_size() > 0)
+        assert p._backend.compute_batch_size() > 0
 
 
 def test_exception_dispatch():
     "Make sure that exception raised during dispatch are indeed captured"
-    assert_raises(
-        ValueError,
-        Parallel(n_jobs=2, pre_dispatch=16, verbose=0),
-        (delayed(exception_raiser)(i) for i in range(30)))
+    with raises(ValueError):
+        Parallel(n_jobs=2, pre_dispatch=16, verbose=0)(
+            delayed(exception_raiser)(i) for i in range(30))
 
 
 def test_nested_exception_dispatch():
     # Ensure TransportableException objects for nested joblib cases gets
     # propagated.
-    assert_raises(
-        JoblibException,
-        Parallel(n_jobs=2, pre_dispatch=16, verbose=0),
-        (delayed(SafeFunction(exception_raiser))(i) for i in range(30)))
+    with raises(JoblibException):
+        Parallel(n_jobs=2, pre_dispatch=16, verbose=0)(
+            delayed(SafeFunction(exception_raiser))(i) for i in range(30))
 
 
 def _reload_joblib():
@@ -486,9 +437,10 @@ def test_multiple_spawning():
     # subprocesses will raise an error, to avoid infinite loops on
     # systems that do not support fork
     if not int(os.environ.get('JOBLIB_MULTIPROCESSING', 1)):
-        raise nose.SkipTest()
-    assert_raises(ImportError, Parallel(n_jobs=2, pre_dispatch='all'),
-                  [delayed(_reload_joblib)() for i in range(10)])
+        raise SkipTest()
+    with raises(ImportError):
+        Parallel(n_jobs=2, pre_dispatch='all')(
+            [delayed(_reload_joblib)() for i in range(10)])
 
 
 class FakeParallelBackend(SequentialBackend):
@@ -506,74 +458,73 @@ class FakeParallelBackend(SequentialBackend):
 
 
 def test_invalid_backend():
-    assert_raises(ValueError, Parallel, backend='unit-testing')
+    with raises(ValueError):
+        Parallel(backend='unit-testing')
 
 
 def test_register_parallel_backend():
     try:
         register_parallel_backend("test_backend", FakeParallelBackend)
-        assert_true("test_backend" in BACKENDS)
-        assert_equal(BACKENDS["test_backend"], FakeParallelBackend)
+        assert "test_backend" in BACKENDS
+        assert BACKENDS["test_backend"] == FakeParallelBackend
     finally:
         del BACKENDS["test_backend"]
 
 
 def test_overwrite_default_backend():
-    assert_equal(_active_backend_type(), MultiprocessingBackend)
+    assert _active_backend_type() == MultiprocessingBackend
     try:
         register_parallel_backend("threading", BACKENDS["threading"],
                                   make_default=True)
-        assert_equal(_active_backend_type(), ThreadingBackend)
+        assert _active_backend_type() == ThreadingBackend
     finally:
         # Restore the global default manually
         parallel.DEFAULT_BACKEND = 'multiprocessing'
-    assert_equal(_active_backend_type(), MultiprocessingBackend)
+    assert _active_backend_type() == MultiprocessingBackend
 
 
 def check_backend_context_manager(backend_name):
     with parallel_backend(backend_name, n_jobs=3):
         active_backend, active_n_jobs = parallel.get_active_backend()
-        assert_equal(active_n_jobs, 3)
-        assert_equal(effective_n_jobs(3), 3)
+        assert active_n_jobs == 3
+        assert effective_n_jobs(3) == 3
         p = Parallel()
-        assert_equal(p.n_jobs, 3)
+        assert p.n_jobs == 3
         if backend_name == 'multiprocessing':
-            assert_equal(type(active_backend), MultiprocessingBackend)
-            assert_equal(type(p._backend), MultiprocessingBackend)
+            assert type(active_backend) == MultiprocessingBackend
+            assert type(p._backend) == MultiprocessingBackend
         elif backend_name == 'threading':
-            assert_equal(type(active_backend), ThreadingBackend)
-            assert_equal(type(p._backend), ThreadingBackend)
+            assert type(active_backend) == ThreadingBackend
+            assert type(p._backend) == ThreadingBackend
         elif backend_name.startswith('test_'):
-            assert_equal(type(active_backend), FakeParallelBackend)
-            assert_equal(type(p._backend), FakeParallelBackend)
+            assert type(active_backend) == FakeParallelBackend
+            assert type(p._backend) == FakeParallelBackend
 
 
-def test_backend_context_manager():
-    all_test_backends = ['test_backend_%d' % i for i in range(3)]
-    for test_backend in all_test_backends:
-        register_parallel_backend(test_backend, FakeParallelBackend)
-    all_backends = ['multiprocessing', 'threading'] + all_test_backends
+all_backends_for_context_manager = ['multiprocessing', 'threading'] + \
+                                   ['test_backend_%d' % i for i in range(3)]
 
-    try:
-        assert_equal(_active_backend_type(), MultiprocessingBackend)
-        # check that this possible to switch parallel backends sequentially
-        for test_backend in all_backends:
-            yield check_backend_context_manager, test_backend
 
-        # The default backend is retored
-        assert_equal(_active_backend_type(), MultiprocessingBackend)
+@with_multiprocessing
+@parametrize('backend', all_backends_for_context_manager)
+def test_backend_context_manager(monkeypatch, backend):
+    if backend not in BACKENDS:
+        monkeypatch.setitem(BACKENDS, backend, FakeParallelBackend)
 
-        # Check that context manager switching is thread safe:
-        Parallel(n_jobs=2, backend='threading')(
-            delayed(check_backend_context_manager)(b)
-            for b in all_backends if not b)
+    assert _active_backend_type() == MultiprocessingBackend
+    # check that this possible to switch parallel backends sequentially
+    check_backend_context_manager(backend)
 
-        # The default backend is again retored
-        assert_equal(_active_backend_type(), MultiprocessingBackend)
-    finally:
-        for backend_name in list(BACKENDS.keys()):
-            if backend_name.startswith('test_'):
-                del BACKENDS[backend_name]
+    # The default backend is retored
+    assert _active_backend_type() == MultiprocessingBackend
+
+    # Check that context manager switching is thread safe:
+    Parallel(n_jobs=2, backend='threading')(
+        delayed(check_backend_context_manager)(b)
+        for b in all_backends_for_context_manager if not b)
+
+    # The default backend is again retored
+    assert _active_backend_type() == MultiprocessingBackend
 
 
 class ParameterizedParallelBackend(SequentialBackend):
@@ -585,46 +536,44 @@ class ParameterizedParallelBackend(SequentialBackend):
         self.param = param
 
 
-def test_parameterized_backend_context_manager():
-    register_parallel_backend('param_backend', ParameterizedParallelBackend)
-    try:
-        assert_equal(_active_backend_type(), MultiprocessingBackend)
+def test_parameterized_backend_context_manager(monkeypatch):
+    monkeypatch.setitem(BACKENDS, 'param_backend',
+                        ParameterizedParallelBackend)
+    assert _active_backend_type() == MultiprocessingBackend
 
-        with parallel_backend('param_backend', param=42, n_jobs=3):
-            active_backend, active_n_jobs = parallel.get_active_backend()
-            assert_equal(type(active_backend), ParameterizedParallelBackend)
-            assert_equal(active_backend.param, 42)
-            assert_equal(active_n_jobs, 3)
-            p = Parallel()
-            assert_equal(p.n_jobs, 3)
-            assert_true(p._backend is active_backend)
-            results = p(delayed(sqrt)(i) for i in range(5))
-        assert_equal(results, [sqrt(i) for i in range(5)])
+    with parallel_backend('param_backend', param=42, n_jobs=3):
+        active_backend, active_n_jobs = parallel.get_active_backend()
+        assert type(active_backend) == ParameterizedParallelBackend
+        assert active_backend.param == 42
+        assert active_n_jobs == 3
+        p = Parallel()
+        assert p.n_jobs == 3
+        assert p._backend is active_backend
+        results = p(delayed(sqrt)(i) for i in range(5))
+    assert results == [sqrt(i) for i in range(5)]
 
-        # The default backend is again retored
-        assert_equal(_active_backend_type(), MultiprocessingBackend)
-    finally:
-        del BACKENDS['param_backend']
+    # The default backend is again restored
+    assert _active_backend_type() == MultiprocessingBackend
 
 
 def test_direct_parameterized_backend_context_manager():
-    assert_equal(_active_backend_type(), MultiprocessingBackend)
+    assert _active_backend_type() == MultiprocessingBackend
 
     # Check that it's possible to pass a backend instance directly,
     # without registration
     with parallel_backend(ParameterizedParallelBackend(param=43), n_jobs=5):
         active_backend, active_n_jobs = parallel.get_active_backend()
-        assert_equal(type(active_backend), ParameterizedParallelBackend)
-        assert_equal(active_backend.param, 43)
-        assert_equal(active_n_jobs, 5)
+        assert type(active_backend) == ParameterizedParallelBackend
+        assert active_backend.param == 43
+        assert active_n_jobs == 5
         p = Parallel()
-        assert_equal(p.n_jobs, 5)
-        assert_true(p._backend is active_backend)
+        assert p.n_jobs == 5
+        assert p._backend is active_backend
         results = p(delayed(sqrt)(i) for i in range(5))
-    assert_equal(results, [sqrt(i) for i in range(5)])
+    assert results == [sqrt(i) for i in range(5)]
 
     # The default backend is again retored
-    assert_equal(_active_backend_type(), MultiprocessingBackend)
+    assert _active_backend_type() == MultiprocessingBackend
 
 
 ###############################################################################
@@ -640,41 +589,39 @@ def test_joblib_exception():
 
 def test_safe_function():
     safe_division = SafeFunction(division)
-    assert_raises(JoblibException, safe_division, 1, 0)
+    with raises(JoblibException):
+        safe_division(1, 0)
 
 
-def test_invalid_batch_size():
-    assert_raises(ValueError, Parallel, batch_size=0)
-    assert_raises(ValueError, Parallel, batch_size=-1)
-    assert_raises(ValueError, Parallel, batch_size=1.42)
+@parametrize('batch_size', [0, -1, 1.42])
+def test_invalid_batch_size(batch_size):
+    with raises(ValueError):
+        Parallel(batch_size=batch_size)
 
 
-def check_same_results(params):
-    n_tasks = params.pop('n_tasks')
-    expected = [square(i) for i in range(n_tasks)]
-    results = Parallel(**params)(delayed(square)(i) for i in range(n_tasks))
-    assert_equal(results, expected)
-
-
-def test_dispatch_race_condition():
+@parametrize('n_tasks, n_jobs, pre_dispatch, batch_size',
+             [(2, 2, 'all', 'auto'),
+              (2, 2, 'n_jobs', 'auto'),
+              (10, 2, 'n_jobs', 'auto'),
+              (517, 2, 'n_jobs', 'auto'),
+              (10, 2, 'n_jobs', 'auto'),
+              (10, 4, 'n_jobs', 'auto'),
+              (25, 4, '2 * n_jobs', 1),
+              (25, 4, 'all', 1),
+              (25, 4, '2 * n_jobs', 7),
+              (10, 4, '2 * n_jobs', 'auto')])
+def test_dispatch_race_condition(n_tasks, n_jobs, pre_dispatch, batch_size):
     # Check that using (async-)dispatch does not yield a race condition on the
     # iterable generator that is not thread-safe natively.
     # This is a non-regression test for the "Pool seems closed" class of error
-    yield check_same_results, dict(n_tasks=2, n_jobs=2, pre_dispatch="all")
-    yield check_same_results, dict(n_tasks=2, n_jobs=2, pre_dispatch="n_jobs")
-    yield check_same_results, dict(n_tasks=10, n_jobs=2, pre_dispatch="n_jobs")
-    yield check_same_results, dict(n_tasks=517, n_jobs=2,
-                                   pre_dispatch="n_jobs")
-    yield check_same_results, dict(n_tasks=10, n_jobs=2, pre_dispatch="n_jobs")
-    yield check_same_results, dict(n_tasks=10, n_jobs=4, pre_dispatch="n_jobs")
-    yield check_same_results, dict(n_tasks=25, n_jobs=4, batch_size=1)
-    yield check_same_results, dict(n_tasks=25, n_jobs=4, batch_size=1,
-                                   pre_dispatch="all")
-    yield check_same_results, dict(n_tasks=25, n_jobs=4, batch_size=7)
-    yield check_same_results, dict(n_tasks=10, n_jobs=4,
-                                   pre_dispatch="2*n_jobs")
+    params = {'n_jobs': n_jobs, 'pre_dispatch': pre_dispatch,
+              'batch_size': batch_size}
+    expected = [square(i) for i in range(n_tasks)]
+    results = Parallel(**params)(delayed(square)(i) for i in range(n_tasks))
+    assert results == expected
 
 
+@with_multiprocessing
 def test_default_mp_context():
     p = Parallel(n_jobs=2, backend='multiprocessing')
     context = p._backend_args.get('context')
@@ -686,20 +633,20 @@ def test_default_mp_context():
         if env_method is None:
             # Check the default behavior
             if sys.platform == 'win32':
-                assert_equal(start_method, 'spawn')
+                assert start_method == 'spawn'
             else:
-                assert_equal(start_method, 'fork')
+                assert start_method == 'fork'
         else:
-            assert_equal(start_method, env_method)
+            assert start_method == env_method
     else:
-        assert_equal(context, None)
+        assert context is None
 
 
+@with_multiprocessing
 @with_numpy
 def test_no_blas_crash_or_freeze_with_multiprocessing():
     if sys.version_info < (3, 4):
-        raise nose.SkipTest('multiprocessing can cause BLAS freeze on'
-                            ' old Python')
+        raise SkipTest('multiprocessing can cause BLAS freeze on old Python')
 
     # Use the spawn backend that is both robust and available on all platforms
     spawn_backend = mp.get_context('spawn')
@@ -724,17 +671,14 @@ def test_no_blas_crash_or_freeze_with_multiprocessing():
 def test_parallel_with_interactively_defined_functions():
     # When functions are defined interactively in a python/IPython
     # session, we want to be able to use them with joblib.Parallel
-
-    try:
-        import posix
-    except ImportError:
+    if posix is None:
         # This test pass only when fork is the process start method
-        raise nose.SkipTest('Not a POSIX platform')
+        raise SkipTest('Not a POSIX platform')
 
     code = '\n\n'.join([
         'from joblib import Parallel, delayed',
-        'def sqrt(x): return x**2',
-        'print(Parallel(n_jobs=2)(delayed(sqrt)(i) for i in range(5)))'])
+        'def square(x): return x**2',
+        'print(Parallel(n_jobs=2)(delayed(square)(i) for i in range(5)))'])
 
     check_subprocess_call([sys.executable, '-c', code],
                           stdout_regex=r'\[0, 1, 4, 9, 16\]')
@@ -742,7 +686,7 @@ def test_parallel_with_interactively_defined_functions():
 
 def test_parallel_with_exhausted_iterator():
     exhausted_iterator = iter([])
-    assert_equal(Parallel(n_jobs=2)(exhausted_iterator), [])
+    assert Parallel(n_jobs=2)(exhausted_iterator) == []
 
 
 def check_memmap(a):
@@ -768,3 +712,52 @@ def test_auto_memmap_on_arrays_from_generator():
         delayed(check_memmap)(a) for a in generate_arrays(100))
     for result, expected in zip(results, generate_arrays(len(results))):
         np.testing.assert_array_equal(expected, result)
+
+
+@with_multiprocessing
+def test_nested_parallel_warnings(capfd):
+    if posix is None:
+        # This test pass only when fork is the process start method
+        raise SkipTest('Not a POSIX platform')
+
+    # no warnings if inner_n_jobs=1
+    Parallel(n_jobs=2)(delayed(parallel_func)(inner_n_jobs=1)
+                       for _ in range(5))
+    out, err = capfd.readouterr()
+    assert err == ''
+
+    #  warnings if inner_n_jobs != 1
+    Parallel(n_jobs=2)(delayed(parallel_func)(inner_n_jobs=2)
+                       for _ in range(5))
+    out, err = capfd.readouterr()
+    assert 'Multiprocessing-backed parallel loops cannot be nested' in err
+
+
+def identity(arg):
+    return arg
+
+
+@with_numpy
+@with_multiprocessing
+def test_memmap_with_big_offset(tmpdir):
+    fname = tmpdir.join('test.mmap').strpath
+    size = mmap.ALLOCATIONGRANULARITY
+    obj = [np.zeros(size, dtype='uint8'), np.ones(size, dtype='uint8')]
+    dump(obj, fname)
+    memmap = load(fname, mmap_mode='r')
+    result, = Parallel(n_jobs=2)(delayed(identity)(memmap) for _ in [0])
+    assert isinstance(memmap[1], np.memmap)
+    assert memmap[1].offset > size
+    np.testing.assert_array_equal(obj, result)
+
+
+def test_warning_about_timeout_not_supported_by_backend():
+    with warns(None) as warninfo:
+        Parallel(timeout=1)(delayed(square)(i) for i in range(10))
+    assert len(warninfo) == 1
+    w = warninfo[0]
+    assert isinstance(w.message, UserWarning)
+    assert str(w.message) == (
+        "The backend class 'SequentialBackend' does not support timeout. "
+        "You have set 'timeout=1' in Parallel but the 'timeout' parameter "
+        "will not be used.")
