@@ -9,9 +9,20 @@ import re
 import subprocess
 import threading
 
+import pytest
+import _pytest
+
 from joblib._compat import PY3_OR_LATER
 
 
+raises = pytest.raises
+warns = pytest.warns
+SkipTest = _pytest.runner.Skipped
+skipif = pytest.mark.skipif
+fixture = pytest.fixture
+parametrize = pytest.mark.parametrize
+
+
 def warnings_to_stdout():
     """ Redirect all warnings to stdout.
     """
@@ -21,34 +32,7 @@ def warnings_to_stdout():
         showwarning_orig(msg, cat, os.path.basename(fname), line, sys.stdout)
 
     warnings.showwarning = showwarning
-    #warnings.simplefilter('always')
-
-
-try:
-    from nose.tools import assert_raises_regex
-except ImportError:
-    # For Python 2.7
-    try:
-        from nose.tools import assert_raises_regexp as assert_raises_regex
-    except ImportError:
-        # for Python 2.6
-        def assert_raises_regex(expected_exception, expected_regexp,
-                                callable_obj=None, *args, **kwargs):
-            """Helper function to check for message patterns in exceptions"""
-
-            not_raised = False
-            try:
-                callable_obj(*args, **kwargs)
-                not_raised = True
-            except Exception as e:
-                error_message = str(e)
-                if not re.compile(expected_regexp).search(error_message):
-                    raise AssertionError("Error message should match pattern "
-                                         "%r. %r does not." %
-                                         (expected_regexp, error_message))
-            if not_raised:
-                raise AssertionError("Should have raised %r" %
-                                     expected_exception(expected_regexp))
+    # warnings.simplefilter('always')
 
 
 def check_subprocess_call(cmd, timeout=1, stdout_regex=None,
@@ -73,20 +57,20 @@ def check_subprocess_call(cmd, timeout=1, stdout_regex=None,
             stdout, stderr = stdout.decode(), stderr.decode()
         if proc.returncode != 0:
             message = (
-                'Non-zero return code: {0}.\nStdout:\n{1}\n'
-                'Stderr:\n{2}').format(
+                'Non-zero return code: {}.\nStdout:\n{}\n'
+                'Stderr:\n{}').format(
                     proc.returncode, stdout, stderr)
             raise ValueError(message)
 
         if (stdout_regex is not None and
                 not re.search(stdout_regex, stdout)):
             raise ValueError(
-                "Unexpected stdout: {0!r} does not match:\n{1!r}".format(
+                "Unexpected stdout: {!r} does not match:\n{!r}".format(
                     stdout_regex, stdout))
         if (stderr_regex is not None and
                 not re.search(stderr_regex, stderr)):
             raise ValueError(
-                "Unexpected stderr: {0!r} does not match:\n{1!r}".format(
+                "Unexpected stderr: {!r} does not match:\n{!r}".format(
                     stderr_regex, stderr))
 
     finally:
